﻿using Otter;
using System;
using ThatsCrazyWhat.entities.enemies;
using ThatsCrazyWhat.Utility;

namespace ThatsCrazyWhat.entities.bullets
{
    class WaterDrop : Entity
    {
        private double angleInRadians = 1;
        private Spritemap<string> _spriteMap;
        private int life = 150;//this number should be just slightly longer than their detection range for shooting.

        public WaterDrop(float x, float y, double destiny)
        {
            X = x; Y = y;
            _spriteMap = new Spritemap<string>(Assets.WATERDROP, 10, 10);
            _spriteMap.Add("spin", new int[] { 0, 1, 2, 3, 2, 1 }, new float[] { 5f, 5f });
            SetHitbox(4, 4, Global.Type.WATERDROP);
            Collider.X = -2;
            Collider.Y = -2;
            _spriteMap.CenterOrigin();
            _spriteMap.Play("spin");
            Graphic = _spriteMap;
            angleInRadians = destiny;
            SoundManager.PlayRequestedSound("_splashShot");
        }

        public override void Update()
        {
            base.Update();
            X -= Global.PlayerFiredWaterSpeed * (float)Math.Cos(angleInRadians);
            Y -= Global.PlayerFiredWaterSpeed * (float)Math.Sin(angleInRadians);
            if (life <= 0)
                Destroy();
            if (life > 0)
                life--;
            HandleCollisions();
        }//end of update

        public void HandleCollisions()//lets try this. I hope this works. It isn't. :O
        {
            var colt = Collider.Collide((int)X, (int)Y, Global.Type.AGENT);
            if (colt != null)
            {
                Agent t = (Agent)colt.Entity;
                t.Destroy();
                //play destruction sound here;
                Destroy();
            }

            var colg = Collider.Collide((int)X, (int)Y, Global.Type.CIVILIAN);
            var colgr = Collider.Collide((int)X, (int)Y, Global.Type.RUNNINGCIVILIAN);
            if (colg != null || colgr != null)
            {
                Civilian tr = (Civilian)colg.Entity;
                tr.Destroy();
                //play destruction sound here;
                Destroy();
            }

            /*var colb = Collider.Collide((int)X, (int)Y, Global.Type.BOSS);
            if (colb != null)
            {
                SoundManager.PlayRequestedSound("_bossHurt");
                Global.BossHealth--;
                Destroy();
            }*/
        }

        public void Destroy()
        {
            SoundManager.PlayRequestedSound("_waterHit");
            RemoveSelf();
        }

        public override void Render()
        {
            base.Render();
            //Collider.Render(Color.Green);
        }
    }
}
