﻿using Otter;
using ThatsCrazyWhat.effects;
using ThatsCrazyWhat.entities;
using ThatsCrazyWhat.objects;
using ThatsCrazyWhat.ui;
using ThatsCrazyWhat.Utility;
using TiledSharp;

namespace ThatsCrazyWhat.scenes.Levels
{
    class Hanger : Scene
    {
        public const int _width = Global.MW * 3;
        public const int _height = Global.MH * 2;
        private bool _addGlobalUI = true;
        private bool one;
        private bool _addHealth;
        private bool activateFirstFoodItem;
        private bool _addedSecondFoodItem;
        private bool _addedThirdFoodItem;
        private bool _addedExitStrategy;

        #region level 01
        public TmxMap tmxMap = new TmxMap(Assets.LEVELONE);
        public GridCollider grid = null;

        public Tilemap layerOne = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerTwo = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerThree = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerFour = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        private bool _addedFourthFoodItem;
        #endregion

        public Hanger(string doorCameIn = "")
        {
            #region MapCreation 01
            grid = new GridCollider(_width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT, (int)Global.Type.SOLID);
            Entity gridEntity = new Entity(0, 0, null, grid);
            Add(gridEntity);

            layerOne.DefaultLayerName = tmxMap.Layers[3].Name;
            layerTwo.DefaultLayerName = tmxMap.Layers[2].Name;
            layerThree.DefaultLayerName = tmxMap.Layers[1].Name;
            layerFour.DefaultLayerName = tmxMap.Layers[0].Name;

            for (int z = 0; z < tmxMap.Layers.Count; z++)
            {
                TmxLayer layer = tmxMap.Layers[z];

                if (layer.Name == "Layer 1")
                    layerOne.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else if (layer.Name == "Layer 2")
                    layerTwo.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else if (layer.Name == "Layer 3")
                    layerThree.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else
                    layerFour.AddLayer(layer.Name, tmxMap.Layers.Count - z);

                for (int x = 0; x < tmxMap.Width; x++)
                {
                    for (int y = 0; y < tmxMap.Height; y++)
                    {
                        int i = y * tmxMap.Width + x;
                        if (layer.Name == "Layer 1")//walls
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerOne.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                            if (layer.Tiles[i].Gid != 0)
                                grid.SetTile(x, y, true);
                        }
                        else if (layer.Name == "Layer 2")//main tiles
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerTwo.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                        }
                        else if (layer.Name == "Layer 3")//light blue water
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerThree.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                        }
                        else//dark blue water
                            layerFour.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                    }
                }
            }
            #endregion
            AddGraphic(layerFour);
            AddGraphic(layerThree);
            AddGraphic(layerTwo);
            AddGraphic(layerOne);

            Global.cam = new Camera(100, 2);
            #region Add Players and UI
            if (doorCameIn == "leftDoor")
                Global.player = new Player(1375, 339);
            else
                Global.player = new Player(Global.MW / 2, Global.MH / 2 - 200);
            Add(Global.player);
            //Add(new Score(16));
            Add(new PlayerHealth(5, 440));
            Global.WriteWord(4 - 1, 440 - 1, "Health:", Color.Red, 1f, -101, this, 0, 10);//regular text  
            Global.WriteWord(4, 440, "Health:", Color.Black, 1f, -100, this, 0, 10);//shadow (layers sort this)
            Add(new WaterBulletsRemaining(560, 10));
            Global.WriteWord(550 - 1, 10 - 1, "WaterDrops:", Color.FromBytes(41, 182, 246, 255), 1f, -101, this, 0, 10);//regular text  
            Global.WriteWord(550, 10, "WaterDrops:", Color.Black, 1f, -100, this, 0, 10);//shadow (layers sort this)
            if (Global.FirstFoodItem && !one)
            {
                one = true;
                Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "1 Crate", Color.FromBytes(168, 112, 64, 255), 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "1 Crate", Color.Black, 1f, -100, this, 0, 10);
            }
            #endregion
            if (Global.IsCartBeingPushed)
                Add(new Cart(1375, 339));
            else if (Global.MemoryCartX != 0 && Global.CartIsOnThisMap == "Hanger")
                Add(new Cart(Global.MemoryCartX, Global.MemoryCartY));

            for (var i = 0; i < Global.MW; i += 100)
            {
                Add(new LightingInHangar(i, 0));
            }
            Add(new HangerSideDoor(1408, 352));

            #region planes
            Add(new WorstPlaneEver(100, 100));
            Add(new WorstPlaneEver(400, 100));
            Add(new WorstPlaneEver(900, 100));
            Add(new WorstPlaneEver(1200, 100));
            Add(new WorstPlaneEver(100, 400));
            Add(new WorstPlaneEver(100, 650));
            Add(new WorstPlaneEver(850, 450));
            Add(new WorstPlaneEver(950, 700));
            #endregion
            if (!Global.IsUFOCrashed)//if it's not crashed play the entire thing, otherwise it's already down in the middle of the place
                Add(new SpaceShip(Global.MW, -50));
            else if (!Global.GetOut)
            {
                Add(new SpaceShip(626, 388));
                Add(new PersonOfInterest(626 + 110, 388 + 60));
                Add(new AnneBelleHealth(300, 440));
                Global.WriteWord(260 - 1, 440 - 1, "Alien Health:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);//regular text  
                Global.WriteWord(260, 440, "Alien Health:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);//shadow (layers sort this)
            }
            else
            {
                Add(new AnneBelle(1375, 339));
                Add(new CountDownTimerDisplay(400, 10));
            }
            if (Global.PickedUpCrate && Global.FirstFoodItem)
            {
                Add(new Crate(Global.PlayerX, Global.PlayerY, true));
            }
            if (Global.PickedUpPresident && Global.SecondFoodItem)
            {
                Add(new President(Global.PlayerX, Global.PlayerY, true));
            }
            if (Global.PickedUpMicrowave && Global.ThirdFoodItem)
            {
                Add(new Microwave(Global.PlayerX, Global.PlayerY, true));
            }
            if (Global.PickedUpPlane && Global.FourthFoodItem)
            {
                Add(new WorstPlaneEver(Global.PlayerX, Global.PlayerY, true));
            }
            /*if (Global.music == null)
                Global.music = new Music(Assets.MUSIC_STAGE_THEME);
            if (!Global.music.IsPlaying)
            {
                Global.music.Play();
                Global.music.Volume = .3f;
                Global.music.Loop = true;
            }*/
            Add(Global.cam);
            //for (var i = 0; i < 3; i++)
            //{
            //    Add(new Robot(Rand.Int(16, Global.MW - 16), Rand.Int(16, Global.MH - 32)));
            //}

            //always add this guy so we can pause the game:
            Add(new PausePerson());
        }

        public override void Begin()
        {
            base.Begin();
            Global.cam.adjustToPlayer(Global.MW, Global.MH, Global.PlayerX, Global.PlayerY);
        }
        public override void Update()
        {
            base.Update();
            if (!IsGroupPaused(0))
            {
                if (_addGlobalUI)
                {
                    _addGlobalUI = false;
                    //Global.AddAllLevelUI(Global.GAME.Scene);//so i don't have to add the same ui every level if all levels have it.
                }
                Shared.LevelUpdate(Global.GAME.Scene);
                //all other unique level updates go here:
                if (Global.AnneBelleHealth <= 0 || Global.PlayerHealth <= 0)
                {
                    Global.GAME.RemoveScene();
                    Global.GAME.AddScene(new LoseScreen());// Add next level here or, where you want to go
                }
            }
            if (Global.IsUFOCrashed && !_addHealth && !Global.GetOut)//if it's not crashed play the entire thing, otherwise it's already down in the middle of the place
            {
                if (Global.FirstFoodItem && !activateFirstFoodItem)
                {
                    activateFirstFoodItem = true;
                    Global.FirstFoodItem = true;
                }
                _addHealth = true;
                Add(new AnneBelleHealth(300, 440));
                Global.WriteWord(260 - 1, 440 - 1, "Alien Health:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);//regular text  
                Global.WriteWord(260, 440, "Alien Health:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);//shadow (layers sort this)
                if (Global.FirstFoodItem)
                {
                    Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                    Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                    Global.WriteWordUI(100 - 1, 10 - 1, "1 Crate", Color.FromBytes(168, 112, 64, 255), 1f, -101, this, 0, 10);
                    Global.WriteWordUI(100, 10, "1 Crate", Color.Black, 1f, -100, this, 0, 10);
                }
            }
            if (Global.SecondFoodItem && !_addedSecondFoodItem)
            {
                _addedSecondFoodItem = true;
                RemoveText();
                Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "1 President", Color.Red, 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "1 President", Color.Black, 1f, -100, this, 0, 10);
                Global.RightDoorActive = true;
            }
            if (Global.ThirdFoodItem && !_addedThirdFoodItem)
            {
                _addedThirdFoodItem = true;
                RemoveText();
                Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "1 Microwave", Color.Gold, 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "1 Microwave", Color.Black, 1f, -100, this, 0, 10);
                Global.RightDoorActive = true;
            }
            if (Global.FourthFoodItem && !_addedFourthFoodItem)
            {
                _addedFourthFoodItem = true;
                RemoveText();
                Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "1 Jet Airplane", Color.White, 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "1 Jet Airplane", Color.Black, 1f, -100, this, 0, 10);
                Global.IsPlaneActive = true;
            }
            if (Global.GetOut && !_addedExitStrategy)
            {
                _addedExitStrategy = true;
                RemoveText();
                Global.WriteWord(10 - 1, 10 - 1, "Lead Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Lead Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "Get Out; Find Her Dropship Zone!", Color.Cyan, 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "Get Out; Find Her Dropship Zone!", Color.Black, 1f, -100, this, 0, 10);
                Global.LeftLowerDoorActive = true;
                Add(new CountDownTimerDisplay(350, 10));
            }
        }//end of UPDATE

        public void RemoveText()
        {
            var anyNumbersUI = GetEntities<NumbersUI>();
            foreach (var item in anyNumbersUI)
            {
                item.RemoveSelf();
            }
        }
        public override void Render()
        {
            base.Render();
            //grid.Render();
        }
    }
}
