﻿using Otter;
using ThatsCrazyWhat.entities;
using ThatsCrazyWhat.entities.enemies;
using ThatsCrazyWhat.objects;
using ThatsCrazyWhat.ui;
using ThatsCrazyWhat.Utility;
using TiledSharp;

namespace ThatsCrazyWhat.scenes.Levels
{
    class Rally : Scene
    {
        public const int _width = Global.MW * 3;
        public const int _height = Global.MH * 2;
        private bool _addGlobalUI = true;

        #region level 01
        public TmxMap tmxMap = new TmxMap(Assets.RALLY);
        public GridCollider grid = null;

        public Tilemap layerOne = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerTwo = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerThree = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        public Tilemap layerFour = new Tilemap(Assets.TILEFILE, _width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT);
        private bool one;

        public bool AddedNextItem { get; private set; }
        #endregion

        public Rally(string doorCameIn = "")
        {
            #region MapCreation 01
            grid = new GridCollider(_width, _height, Global.GRID_WIDTH, Global.GRID_HEIGHT, (int)Global.Type.SOLID);
            Entity gridEntity = new Entity(0, 0, null, grid);
            Add(gridEntity);

            layerOne.DefaultLayerName = tmxMap.Layers[3].Name;
            layerTwo.DefaultLayerName = tmxMap.Layers[2].Name;
            layerThree.DefaultLayerName = tmxMap.Layers[1].Name;
            layerFour.DefaultLayerName = tmxMap.Layers[0].Name;

            for (int z = 0; z < tmxMap.Layers.Count; z++)
            {
                TmxLayer layer = tmxMap.Layers[z];

                if (layer.Name == "Layer 1")
                    layerOne.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else if (layer.Name == "Layer 2")
                    layerTwo.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else if (layer.Name == "Layer 3")
                    layerThree.AddLayer(layer.Name, tmxMap.Layers.Count - z);
                else
                    layerFour.AddLayer(layer.Name, tmxMap.Layers.Count - z);

                for (int x = 0; x < tmxMap.Width; x++)
                {
                    for (int y = 0; y < tmxMap.Height; y++)
                    {
                        int i = y * tmxMap.Width + x;
                        if (layer.Name == "Layer 1")//walls
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerOne.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                            if (layer.Tiles[i].Gid != 0)
                                grid.SetTile(x, y, true);
                        }
                        else if (layer.Name == "Layer 2")//main tiles
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerTwo.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                        }
                        else if (layer.Name == "Layer 3")//light blue water
                        {
                            if (layer.Tiles[i].Gid != 0)
                                layerThree.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                        }
                        else//dark blue water
                            layerFour.SetTile(x, y, layer.Tiles[i].Gid - 1, layer.Name);
                    }
                }
            }
            #endregion
            AddGraphic(layerFour);
            AddGraphic(layerThree);
            AddGraphic(layerTwo);
            AddGraphic(layerOne);

            Global.cam = new Camera(100, 2);
            #region Add Players and UI
            if (doorCameIn == "rightDoor")
            {
                Global.player = new Player(31, 339);
                if (Global.GetOut)
                {
                    Add(new AnneBelle(31, 339));
                    Add(new CountDownTimerDisplay(350, 10));
                }
            }

            else
                Global.player = new Player(Global.MW / 2, Global.MH / 2 - 200);
            Add(Global.player);
            Add(new PlayerHealth(5, 440));
            Global.WriteWord(4 - 1, 440 - 1, "Health:", Color.Red, 1f, -101, this, 0, 10);//regular text  
            Global.WriteWord(4, 440, "Health:", Color.Black, 1f, -100, this, 0, 10);//shadow (layers sort this)
            Add(new WaterBulletsRemaining(560, 10));
            Global.WriteWord(550 - 1, 10 - 1, "WaterDrops:", Color.FromBytes(41, 182, 246, 255), 1f, -101, this, 0, 10);//regular text  
            if (Global.SecondFoodItem && !one)
            {
                one = true;
                Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "1 President", Color.Red, 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "1 President", Color.Black, 1f, -100, this, 0, 10);
            }
            if (Global.ThirdFoodItem && !one)
            {
                one = true;
                Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "1 Mircowave", Color.Gold, 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "1 Microwave", Color.Black, 1f, -100, this, 0, 10);
                Global.RightLowerDoorActive = true;
            }
            if (Global.FourthFoodItem && !one)
            {
                one = true;
                Global.WriteWord(10 - 1, 10 - 1, "Feed Her:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);
                Global.WriteWord(10, 10, "Feed Her:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);
                Global.WriteWordUI(100 - 1, 10 - 1, "1 Jet Airplane", Color.White, 1f, -101, this, 0, 10);
                Global.WriteWordUI(100, 10, "1 Jet Airplane", Color.Black, 1f, -100, this, 0, 10);
            }
            #endregion
            if (Global.IsCartBeingPushed)
                Add(new Cart(31, 338));
            else if (Global.MemoryCartX != 0 && Global.CartIsOnThisMap == "Rally")
                Add(new Cart(Global.MemoryCartX, Global.MemoryCartY));

            Add(new RallySideDoor(-16, 352));

            Add(new AnneBelleHealth(300, 440));
            Global.WriteWord(260 - 1, 440 - 1, "Alien Health:", Color.FromBytes(216, 169, 224, 255), 1f, -101, this, 0, 10);//regular text  
            Global.WriteWord(260, 440, "Alien Health:", Color.FromBytes(162, 207, 110, 255), 1f, -100, this, 0, 10);//shadow (layers sort this)
            if (Global.GetOut && !one)
            {
                one = false;
                for (var i = 0; i < 50; i++)
                {
                    Add(new Agent(Rand.Int(39, 322), Rand.Int(244, 380)));
                }
            }
            if (Global.SecondFoodItem)
                Add(new President(220, 240));
            Add(new Podium(220, 270));
            if (Global.PickedUpPlane && Global.FourthFoodItem)
            {
                Add(new WorstPlaneEver(Global.PlayerX, Global.PlayerY, true));
            }
            for (var ii = 0; ii < 200; ii++)
            {
                Add(new Civilian(Rand.Int(38, 311), Rand.Int(480, 800)));
            }
            /*if (Global.music == null)
                Global.music = new Music(Assets.MUSIC_STAGE_THEME);
            if (!Global.music.IsPlaying)
            {
                Global.music.Play();
                Global.music.Volume = .3f;
                Global.music.Loop = true;
            }*/
            Add(Global.cam);
            //for (var i = 0; i < 3; i++)
            //{
            //    Add(new Robot(Rand.Int(16, Global.MW - 16), Rand.Int(16, Global.MH - 32)));
            //}

            //always add this guy so we can pause the game:
            Add(new PausePerson());
        }

        public override void Begin()
        {
            base.Begin();
            Global.cam.adjustToPlayer(Global.MW, Global.MH, Global.PlayerX, Global.PlayerY);
        }
        public override void Update()
        {
            base.Update();
            if (!IsGroupPaused(0))
            {
                if (_addGlobalUI)
                {
                    _addGlobalUI = false;
                    //Global.AddAllLevelUI(Global.GAME.Scene);//so i don't have to add the same ui every level if all levels have it.
                }
                Shared.LevelUpdate(Global.GAME.Scene);
                //all other unique level updates go here:
                if (Global.AnneBelleHealth <= 0 || Global.PlayerHealth <= 0)
                {
                    Global.GAME.RemoveScene();
                    Global.GAME.AddScene(new LoseScreen());// Add next level here or, where you want to go
                }
            }
        }//end of UPDATE

        public void RemoveText()
        {
            var anyNumbersUI = GetEntities<NumbersUI>();
            foreach (var item in anyNumbersUI)
            {
                item.RemoveSelf();
            }
        }
        public override void Render()
        {
            base.Render();
            //grid.Render();
        }
    }
}
